Package: MatrixExtra
Type: Package
Title: Extra Methods for Sparse Matrices
Version: 0.1.10
Authors@R: c(
  person(given="David", family="Cortes", role=c("aut", "cre", "cph"),
         email="david.cortes.rivera@gmail.com"),
  person(given="Dmitry", family="Selivanov", role="cph"),
  person(given="Thibaut", family="Goetghebuer-Planchon", role="cph",
         comment="Copyright holder of included robinmap library"),
  person(given="Martin", family="Maechler", role="cph",
         comment="Copyright holder of 'Matrix' package from which some code was taken"),
  person(given="Robert", family="Gentleman", role="cph",
         comment="Copyright holder of mathematical functions used by base R which were copied"),
  person(given="Ross", family="Ihaka", role="cph",
         comment="Copyright holder of mathematical functions used by base R which were copied")
  )
Maintainer: David Cortes <david.cortes.rivera@gmail.com>
Description: Extends sparse matrix and vector classes from the 'Matrix' package by providing: 
  (a) Methods and operators that work natively on CSR formats (compressed sparse row, 
  a.k.a. 'RsparseMatrix') such as slicing/sub-setting, assignment, rbind(), 
  mathematical operators for CSR and COO such as addition ("+") or sqrt(), and methods such as diag(); 
  (b) Multi-threaded matrix multiplication and cross-product for many <sparse, dense> types, 
  including the 'float32' type from 'float'; 
  (c) Coercion methods between pairs of classes which are not present in 'Matrix', 
  such as 'dgCMatrix' -> 'ngRMatrix', as well as convenience conversion functions; 
  (d) Utility functions for sparse matrices such as sorting the indices or removing 
  zero-valued entries; 
  (e) Fast transposes that work by outputting in the opposite storage format;
  (f) Faster replacements for many 'Matrix' methods for all sparse types, such as
  slicing and elementwise multiplication.
  (g) Convenience functions for sparse objects, such as 'mapSparse' or a shorter 'show' method.
License: GPL (>= 2)
Depends: Matrix (>= 1.3), methods
Imports: Rcpp, RhpcBLASctl, float
LinkingTo: Rcpp, float
Suggests: testthat, data.table, knitr, rmarkdown
VignetteBuilder: knitr
URL: https://github.com/david-cortes/MatrixExtra
BugReports: https://github.com/david-cortes/MatrixExtra/issues
RoxygenNote: 7.1.2
NeedsCompilation: yes
Biarch: TRUE
StagedInstall: TRUE
Packaged: 2022-05-15 17:23:55 UTC; david
Author: David Cortes [aut, cre, cph],
  Dmitry Selivanov [cph],
  Thibaut Goetghebuer-Planchon [cph] (Copyright holder of included
    robinmap library),
  Martin Maechler [cph] (Copyright holder of 'Matrix' package from which
    some code was taken),
  Robert Gentleman [cph] (Copyright holder of mathematical functions used
    by base R which were copied),
  Ross Ihaka [cph] (Copyright holder of mathematical functions used by
    base R which were copied)
Repository: CRAN
Date/Publication: 2022-05-15 18:30:02 UTC
Built: R 4.1.3; x86_64-w64-mingw32; 2022-05-16 07:27:05 UTC; windows
Archs: x64
