# Generated by roxygen2: do not edit by hand

export()
export(as.coo.matrix)
export(as.csc.matrix)
export(as.csr.matrix)
export(as.sparse.vector)
export(check_sparse_matrix)
export(deepcopy_sparse_object)
export(emptySparse)
export(filterSparse)
export(mapSparse)
export(rbind_csr)
export(remove_sparse_zeros)
export(restore_old_matrix_behavior)
export(set_new_matrix_behavior)
export(sort_sparse_indices)
export(t_deep)
export(t_shallow)
exportMethods("%%")
exportMethods("%*%")
exportMethods("%/%")
exportMethods("&")
exportMethods("*")
exportMethods("+")
exportMethods("-")
exportMethods("/")
exportMethods("[")
exportMethods("[<-")
exportMethods("^")
exportMethods("diag<-")
exportMethods("|")
exportMethods(abs)
exportMethods(atanh)
exportMethods(cbind2)
exportMethods(ceiling)
exportMethods(crossprod)
exportMethods(diag)
exportMethods(expm1)
exportMethods(floor)
exportMethods(log1p)
exportMethods(norm)
exportMethods(print)
exportMethods(rbind2)
exportMethods(round)
exportMethods(show)
exportMethods(sign)
exportMethods(signif)
exportMethods(sin)
exportMethods(sinh)
exportMethods(sqrt)
exportMethods(t)
exportMethods(tan)
exportMethods(tanh)
exportMethods(tanpi)
exportMethods(tcrossprod)
exportMethods(trunc)
import(Matrix)
import(methods)
importClassesFrom(float,float32)
importFrom(Rcpp,evalCpp)
importFrom(RhpcBLASctl,blas_get_num_procs)
importFrom(RhpcBLASctl,blas_set_num_threads)
importFrom(float,dbl)
importFrom(parallel,detectCores)
useDynLib(MatrixExtra, .registration=TRUE)
